/****************************************************************************/
/*                                                                          */
/*                   Copyright 2006 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : Main.java                                                */
/* DESCRIPTION   : Top-level driver for Functional Language with Sessions   */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 22nd December 2005                                       */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 4th January 2006                                         */
/****************************************************************************/

package fls;

import fls.node.*;
import fls.lexer.*;
import fls.parser.*;
import fls.error.*;
import fls.checker.*;
import fls.types.*;
import fls.env.*;
 
import java.io.*;
import java.util.*;

public class Main {

    private static Lexer scanner;
    private static Parser parser;

    /** The AST representing the source program. */
    private static Node theAST;

    static void check (String sourceName) {
	try {
	    scanner = new Lexer(new PushbackReader(
		      new BufferedReader(new FileReader(sourceName)), 1024));
	} catch(FileNotFoundException e)
	    {System.out.println("Can't access source file " + sourceName);
	    System.exit(1);};

	parser = new Parser(scanner);
        try {
	    theAST = parser.parse();
	} catch(Exception e)
	    {System.out.println(e);
	     System.exit(1);}

	TypeNameCollector collect = new TypeNameCollector();
	theAST.apply(collect);
	ErrorTable errorTable = collect.getErrorTable();
	if (errorTable.hasErrors()) {
	    errorTable.output(System.out,
			      "while processing " + sourceName);
	    System.exit(1);
	};

	Checker chk = new Checker();
	chk.setTypeNames(collect.getTypeNames());
	theAST.apply(chk);

	errorTable = chk.getErrorTable();
	Env env = chk.getEnv();
	if (env.linearRemaining()) {
	    errorTable.add(-1,new LinVarsUnusedError());
	};
	if (!((Type)chk.getOut(theAST)).unlimited()) {
	    errorTable.add(-1,new LinearProgramTypeError());
	};
	if (errorTable.hasErrors()) {
	    errorTable.output(System.out,
			      "while processing " + sourceName);
	    System.exit(1);
	}
	else {
	    System.out.println("Expression has type " +
			       ((Type)chk.getOut(theAST)).name());
			       } 
    }

    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage: check filename");
            System.exit(1);
        }
        check(args[0]);
    }
}
