/****************************************************************************/
/*                                                                          */
/*                   Copyright 2006 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : ChoiceType.java                                          */
/* DESCRIPTION   : Representation of choice session type                    */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 4th January 2006                                         */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 4th January 2006                                         */
/****************************************************************************/

package fls.types;

import java.util.*;

public class ChoiceType extends Type {
    public Hashtable<String,Type> branches;

    public ChoiceType(Hashtable<String,Type> b) {
	branches = b;
    }

    public boolean unlimited() {
	return false;
    }

    public boolean base() {
	return false;
    }

    public Hashtable<String,Type> getBranches() {
	return branches;
    }

    public String name() {
	String n = "+<";
	Enumeration<String> labels = branches.keys();
	if (labels.hasMoreElements()) {
	    String l = labels.nextElement();
	    Type t = branches.get(l);
	    n = n + l + ":" + t.name();
	};
	while (labels.hasMoreElements()) {
	    String l = labels.nextElement();
	    Type t = branches.get(l);
	    n = n + "," + l + ":" + t.name();
	};
	n = n + ">";
	return n;
    }

    public boolean equal(Type t) {
	if (!(t instanceof ChoiceType))
	    return false;
	if (branches.size() != ((ChoiceType)t).getBranches().size())
	    return false;
	Enumeration<String> labels = branches.keys();
	while (labels.hasMoreElements()) {
	    String l = labels.nextElement();
	    if (!branches.get(l).equal(((ChoiceType)t).getBranches().get(l)))
		return false;
	};
	return true;
    }

    public Type dual() {
	Hashtable<String,Type> br = new Hashtable<String,Type>();
	Enumeration<String> labels = branches.keys();
	while (labels.hasMoreElements()) {
	    String l = labels.nextElement();
	    br.put(l,branches.get(l).dual());
	};
	return new BranchType(br);
    }

}
