/****************************************************************************/
/*                                                                          */
/*                   Copyright 2006 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : NamedType.java                                           */
/* DESCRIPTION   : Representation of named session type                     */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 21st December 2005                                       */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 21st December 2005                                       */
/****************************************************************************/

package fls.types;

import fls.typenv.*;
import fls.error.*;

public class NamedType extends Type {

    public String id;
    public boolean dual;

    public NamedType(String i, boolean d) {
	id = i;
	dual = d;
    }

    public String name() {
	String n = "";
	if (dual) n = "~";
	n = n + id;
	return n;
    }

    public boolean unlimited() {
	return false;
    }

    public boolean base() {
	return true;
    }

    public String getId() {
	return id;
    }

    public boolean getDual() {
	return dual;
    }

    public boolean equal(Type t) {
	return ((t instanceof NamedType)
		&& id.equals(((NamedType)t).getId())
		&& dual == ((NamedType)t).getDual());
    }

    public Type unfold(TypeEnv env) {
	Type ty = env.get(id).getType();
	if (dual) 
	    return ty.dual();
	else
	    return ty;
    }

    public Type dual() {
	return new NamedType(id,!dual);
    }

}
